@@ -7,7 +7,7 @@ use Socket qw(:DEFAULT inet_ntop);
 use Errno;
 use base 'Exporter';
 
-our $VERSION = '0.15';
+our $VERSION = '0.16';
 our @EXPORT_OK = ('connect', 'wrap_connection');
 
 # cache
@@ -179,10 +179,12 @@ sub wrap_connection {
 
 sub _connect {
 	my ($socket, $name, $cfg, $io_socket) = @_;
-	my $ref = ref($socket);
 	
-	if ($socket->isa('IO::Socket::Socks') || !$cfg) {
-		unless ($io_socket and ${*$socket}{'io_socket_timeout'}) {
+	my $ref = ref($socket);
+	my $connected;
+	
+	if ($socket->isa('IO::Socket::Socks') || !$cfg || ( $connected = defined getpeername($socket) )) {
+		unless (!$connected && $io_socket and ${*$socket}{'io_socket_timeout'}) {
 			return CORE::connect( $socket, $name );
 		}
 		
@@ -246,7 +248,7 @@ sub _connect {
 		$socket->blocking(0);
 	}
 	
-	return unless $ok || ($io_handler && $IO::Socket::Socks::SOCKS_ERROR == &IO::Socket::Socks::SOCKS_WANT_WRITE);
+	return unless $ok;
 	bless $socket, $ref if $ref && !$io_handler; # XXX: should we unbless for GLOB?
 	
 	if ($io_handler) {
